/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the Qt Messaging Framework.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QApplication>
#include <QMutex>
#include <QWaitCondition>
#include <qmailnamespace.h>
#include "serverobserver.h"

static void fakeSleep(int time)
{
    QMutex m;
    m.lock();
    QWaitCondition cond;
    cond.wait(&m, time);
}

static bool messageServerRunning()
{
    QString lockfile = "messageserver-instance.lock";
    int lockid = QMail::fileLock(lockfile);
        if (lockid == -1)
                return true;

    QMail::fileUnlock(lockid);
    return false;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    while (!messageServerRunning()) {
        qDebug() << "Message server is not running. Waiting.";
        fakeSleep(5000);
    }

    ServerObserver observer;
    observer.show();
    return app.exec();
}

